//*******************************************************
// Graphic OLED & RTC(RX8900) Test Program ATMega328
// CPU Clock:External 8MHz
//   iq[Yf[^@WȂŁAOUWlgɂj
//
//  file name: disp_ir_nec_data.c
//
//
//  2024/10/21  New (TAB4) Taishi
//*******************************************************

#include	<avr/interrupt.h>
#include	<avr/io.h>

#include	"timer_h.h"
#include	"port_bit_h.h"
#include	"ir_dsm_38khz_h.h"
#include	"oled_1306_h.h"
#include	"glcd_7565_h.h"
#include	"ir_nixie_trnsceiver_h.h"
#include	"ir_nec_trnsceiver_h.h"
#include	"utility_h.h"

#include	"disp_ir_nec_data_h.h"

enum DISP_FUNC_ITEM {		// \֐
		F_6x8 = 0,			// UW\
		F_12x16,			// PQPU\
		F_18x24,			// PWQS\
		F_24x32,			// QSRQ\

		F_END
	};

struct DISP_CONST {
	unsigned char dc_fsize;		//\tHgTCY
	unsigned char dc_x;			//\ʒuw
	unsigned char dc_y;			//\ʒux
	         char *dc_pdt;		//

};

static	unsigned char	sDisp_IC_ypos = 0;
static	unsigned char	sDisp_OLED_wr_ypos = 48;
static	unsigned char	sDisp_GLCD_wr_ypos = 32;
static	unsigned char	sDisp_wr_posx = 0;			// ނwʒu(0--(128-6))


static
struct DISP_CONST	gDisp_block_title[]={
	{F_12x16,   9*6, 0*8, "IR"},
	{F_18x24,   7*6, 2*8+4, "NEC"},
	{F_12x16,   6*6, 6*8, "V1.00"},

	{  F_END,     0,   0,   ""},			// I
};

//*****************************************
// ԊOmdb[h̃^CǧŒ\
//*****************************************
void	disp_ir_nec_mode_title(void)
{
	unsigned char		bn;
	unsigned char		*pbf;
	unsigned char		pos_x;				// hbgPʂ̂wʒu
	unsigned char		pos_y;				// hbgPʂ̂xʒu
	unsigned char		u_func;				// gp֐

	for(bn = 0;  ;bn++){
		if(gDisp_block_title[bn].dc_fsize == F_END){	// tHgTCY eQdlc͕̎\ȂŏI
			break;
		}

		u_func = gDisp_block_title[bn].dc_fsize;	// gp֐̎
		pos_x = gDisp_block_title[bn].dc_x;			// pos_x
		pos_y = gDisp_block_title[bn].dc_y;			// pos_y
		pbf = (unsigned char *)(gDisp_block_title[bn].dc_pdt);			// \

		if(u_func == F_24x32){
			disp_oled_G_24x32(pos_x, pos_y, pbf);	// x, y, data(24x32)
			disp_lcd_G_18x24(pos_x, pos_y, pbf);
		}else if(u_func == F_18x24){
			disp_oled_G_18x24(pos_x, pos_y, pbf);	// x, y, (18x24)
			disp_lcd_G_18x24(pos_x, pos_y, pbf);
		}else if(u_func == F_12x16){
			disp_oled_G_12x16(pos_x, pos_y, pbf);	// x, y, data(12x16)
			disp_lcd_G_12x16(pos_x, pos_y, pbf);
		}else{
			disp_oled_G_6x8(pos_x, pos_y, pbf);			// x, y, data(6x8)
			disp_lcd_G_6x8( pos_x,  pos_y, pbf);
		}
	}
}

//*****************************************
// M̃G[\ nmԖ͂neeԔ͈͊O
//*****************************************
void	proc_disp_err_range_over(unsigned char er, unsigned short us, unsigned short us_min, unsigned short us_max)
{
 	static	char	asc_data[12];

	sDisp_wr_posx = 0;
	sDisp_OLED_wr_ypos += 16;
	if (63 < sDisp_OLED_wr_ypos){
		sDisp_OLED_wr_ypos = 0;
	}
	sDisp_GLCD_wr_ypos += 16;
	if (63 < sDisp_GLCD_wr_ypos){
		sDisp_GLCD_wr_ypos = 0;
	}
	disp_clear_16line_oled_G(sDisp_OLED_wr_ypos);	// 16 Line Clear Display
	disp_clear_16line_lcd_G(sDisp_GLCD_wr_ypos);

	sDisp_IC_ypos += 16;
	if (63 < sDisp_IC_ypos){
		sDisp_IC_ypos = 0;
	}
	set_offse_oled_G(sDisp_IC_ypos);				// OLED scroll
	set_offse_lcd_G(sDisp_IC_ypos);					// GLCD scroll


	wtod_01x(asc_data, er);
	asc_data[1] = '-';
	wtod_04x(&(asc_data[2]), us);
	asc_data[6] = ',';
	wtod_04x(&(asc_data[7]), us_min);
	disp_oled_G_12x16(sDisp_wr_posx, sDisp_OLED_wr_ypos, (unsigned char *)asc_data);
	disp_lcd_G_12x16(sDisp_wr_posx, sDisp_GLCD_wr_ypos, (unsigned char *)asc_data);

}
//*****************************************
// Mf[^̕\@PQPUtHgg
//*****************************************
void	proc_ndisp_rxd_nec(unsigned short us,unsigned char init)
{
// 'us̏ʂWrbgP͓̎ʂɁ@݈ʒuxi{PUjɂāA
//    ݂wʒuԍiOjjɂ
//    x͈ԉ̍siURj𒴂A𒴂玟͈ԏ̍siOjɂ
//
//    \鎞 wʒu iPQV|RUjȏ̎ ʂɁ@݈ʒuxi{PUjɂāA
//    ݂wʒuԍiOjjɂ
//    x͈ԉ̍siURj𒴂A𒴂玟͈ԏ̍siOjɂB
//
//    \ ݂݂̏xʒuƂwʒȕ uc ','{PUiQŏA
//    ݂wʒui{RUjɂBAPԖڂ̃f[^inum == 1)̎','̕ς'>'g
//
//    ŏ݂̏xʒúAnkdcł͂OAfkbcł͂SWɂ
//    ׂ̏ɂ́AnkdcAfkbc̏̒ɕ\hb̕\|C^[
//    ֐KvɂȂ
//    ̓͂rrcPROŨR}h "Set Display Offset" g
//    \点̂łnkdc͏oB
//    Afkbc͓R}h󂯕tȂB
//    悤ȓ "Display start line" ƂR}hB
//    ͕\C͏ɂāAnkdcƓlȕ\ɂȂ
//    Ȃ̂ŁÃR}hgƂɂB
//

	static	unsigned char	sDisp_IC_ypos = 0;
	static	unsigned char	sDisp_OLED_wr_ypos = 48;
	static	unsigned char	sDisp_GLCD_wr_ypos = 32;
	static	unsigned char	sDisp_wr_posx = 0;			// ނwʒu(0--127)
	unsigned char			asc_data[6];
	unsigned char			uc;
	unsigned char			num;

	uc = us & 0xff;
	num = (us >> 8) & 0xff;

	if(init != 0){
		sDisp_IC_ypos = 0;
//		sDisp_OLED_wr_ypos = 48;
		sDisp_OLED_wr_ypos = 56;
//		sDisp_GLCD_wr_ypos = 32;
		sDisp_GLCD_wr_ypos = 40;
		sDisp_wr_posx = 0;
		start_ir_nec_data();
	}else{
		asc_data[0] = ',';
//		if((num == 1)||((127-36) < sDisp_wr_posx)){
		if((num == 1)||((127-18) < sDisp_wr_posx)){
			sDisp_wr_posx = 0;
//			sDisp_OLED_wr_ypos += 16;
			sDisp_OLED_wr_ypos += 8;
			if (63 < sDisp_OLED_wr_ypos){
				sDisp_OLED_wr_ypos = 0;
			}
//			sDisp_GLCD_wr_ypos += 16;
			sDisp_GLCD_wr_ypos += 8;
			if (63 < sDisp_GLCD_wr_ypos){
				sDisp_GLCD_wr_ypos = 0;
			}

			if(num == 1){
				asc_data[0] = '>';
			}

//			disp_clear_16line_oled_G(sDisp_OLED_wr_ypos);	// 16 Line Clear Display
			disp_clear_8line_oled_G(sDisp_OLED_wr_ypos);	// 8 Line Clear Display
//			disp_clear_16line_lcd_G(sDisp_GLCD_wr_ypos);
			disp_clear_8line_lcd_G(sDisp_GLCD_wr_ypos);
//			sDisp_IC_ypos += 16;
			sDisp_IC_ypos += 8;
			if (63 < sDisp_IC_ypos){
				sDisp_IC_ypos = 0;
			}
			set_offse_oled_G(sDisp_IC_ypos);				// OLED scroll
			set_offse_lcd_G(sDisp_IC_ypos);					// GLCD scroll
		}

		wtod_02x((char *)&(asc_data[1]),uc);
//		disp_oled_G_12x16(sDisp_wr_posx, sDisp_OLED_wr_ypos, &(asc_data[0]));
		disp_oled_G_6x8(sDisp_wr_posx, sDisp_OLED_wr_ypos, &(asc_data[0]));
//		disp_lcd_G_12x16(sDisp_wr_posx, sDisp_GLCD_wr_ypos, &(asc_data[0]));
		disp_lcd_G_6x8(sDisp_wr_posx, sDisp_GLCD_wr_ypos, &(asc_data[0]));
//		sDisp_wr_posx += 36;
		sDisp_wr_posx += 18;
	
	}

}

