//*******************************************************
// Graphic OLED & RTC(RX8900) Test Program ATMega328
// CPU Clock:External 8MHz
//   iq[Yf[^@WȂŁAOUWlgɂj
//
//  file name: disp_ir_nixie_data.c
//
//   VAԊOʐMvOɏCB
//
//  2024/10/21  New (TAB4) Taishi
//*******************************************************

#include	<avr/interrupt.h>
#include	<avr/io.h>

#include	"timer_h.h"
#include	"port_bit_h.h"
#include	"ir_dsm_38khz_h.h"
#include	"oled_1306_h.h"
#include	"glcd_7565_h.h"
#include	"ir_nixie_trnsceiver_h.h"

#include	"disp_ir_nixie_data_h.h"

enum DISP_FUNC_ITEM {		// \֐
		F_6x8 = 0,			// UW\
		F_12x16,			// PQPU\
		F_18x24,			// PWQS\
		F_24x32,			// QSRQ\

		F_END
	};

struct DISP_CONST {
	unsigned char dc_fsize;		//\tHgTCY
	unsigned char dc_x;			//\ʒuw
	unsigned char dc_y;			//\ʒux
	         char *dc_pdt;		//

};

static
struct DISP_CONST	gDisp_block_title[]={
	{F_12x16,   9*6, 0*8, "IR"},
	{F_18x24,   4*6, 2*8+4, "NIXIE"},
	{F_12x16,   6*6, 6*8, "V1.00"},

	{  F_END,     0,   0,   ""},			// I
};

//*****************************************
// ԊOjLV[[h̃^CǧŒ\
//*****************************************
void	disp_ir_nixie_mode_title(void)
{
	unsigned char		bn;
	unsigned char		*pbf;
	unsigned char		pos_x;				// hbgPʂ̂wʒu
	unsigned char		pos_y;				// hbgPʂ̂xʒu
	unsigned char		u_func;				// gp֐

	for(bn = 0;  ;bn++){
		if(gDisp_block_title[bn].dc_fsize == F_END){	// tHgTCY eQdlc͕̎\ȂŏI
			break;
		}

		u_func = gDisp_block_title[bn].dc_fsize;	// gp֐̎
		pos_x = gDisp_block_title[bn].dc_x;			// pos_x
		pos_y = gDisp_block_title[bn].dc_y;			// pos_y
		pbf = (unsigned char *)(gDisp_block_title[bn].dc_pdt);			// \

		if(u_func == F_24x32){
			disp_oled_G_24x32(pos_x, pos_y, pbf);	// x, y, data(24x32)
			disp_lcd_G_18x24(pos_x, pos_y, pbf);
		}else if(u_func == F_18x24){
			disp_oled_G_18x24(pos_x, pos_y, pbf);	// x, y, (18x24)
			disp_lcd_G_18x24(pos_x, pos_y, pbf);
		}else if(u_func == F_12x16){
			disp_oled_G_12x16(pos_x, pos_y, pbf);	// x, y, data(12x16)
			disp_lcd_G_12x16(pos_x, pos_y, pbf);
		}else{
			disp_oled_G_6x8(pos_x, pos_y, pbf);			// x, y, data(6x8)
			disp_lcd_G_6x8( pos_x,  pos_y, pbf);
		}
	}
}


//*****************************************
// Mf[^̕\@PQPUtHgg
//*****************************************
void	proc_disp_rxd_nixie(unsigned char uc,unsigned char init)
{
// 's'A݂xʒu̍sĂ
//    \hb̕\|C^[̍si{PUjɂāAUR𒴂OɂB
//    āȀ݂݂xʒuƂwʒȕɏA݂wʒui{PQjɂB
// 'e'A\A݈ʒuxi{PUjɂāA݂wʒuԍiOjjɂ
//    x͈ԉ̍siURj𒴂A𒴂玟͈ԏ̍siOjɂ
//    ŏ݂̏xʒúAnkdcł͂OAfkbcł͂SWɂ
//    ׂ̏ɂ́AnkdcAfkbc̏̒ɕ\hb̕\|C^[
//    ֐KvɂȂ
//    ̓͂rrcPROŨR}h "Set Display Offset" g
//    \点̂łnkdc͏oB
//    Afkbc͓R}h󂯕tȂB
//    悤ȓ "Display start line" ƂR}hB
//    ͕\C͏ɂāAnkdcƓlȕ\ɂȂ
//    Ȃ̂ŁÃR}hgƂɂB
//

	static	unsigned char	sDisp_IC_ypos = 0;
	static	unsigned char	sDisp_OLED_wr_ypos = 48;
	static	unsigned char	sDisp_GLCD_wr_ypos = 32;
	static	unsigned char	sDisp_wr_posx = 0;			// ނwʒu(0--127)

	if(init != 0){
		sDisp_IC_ypos = 0;
		sDisp_OLED_wr_ypos = 0;
		sDisp_GLCD_wr_ypos = 48;
		sDisp_wr_posx = 0;
	}else{
		if(uc == 's'){
			disp_clear_16line_oled_G(sDisp_OLED_wr_ypos);	// 16 Line Clear Display
			disp_clear_16line_lcd_G(sDisp_GLCD_wr_ypos);
			sDisp_IC_ypos += 16;
			if (63 < sDisp_IC_ypos){
				sDisp_IC_ypos = 0;
			}
			set_offse_oled_G(sDisp_IC_ypos);				// OLED scroll
			set_offse_lcd_G(sDisp_IC_ypos);					// GLCD scroll
		}
	
		disp_one_char_oled_G_12x16(sDisp_wr_posx, sDisp_OLED_wr_ypos, uc);
		disp_one_char_lcd_G_12x16(sDisp_wr_posx, sDisp_GLCD_wr_ypos, uc);
		sDisp_wr_posx += 12;
		if (127 < sDisp_wr_posx){
			sDisp_wr_posx = 0;
		}
	
		if(uc == 'e'){
			sDisp_wr_posx = 0;
			sDisp_OLED_wr_ypos += 16;
			if (63 < sDisp_OLED_wr_ypos){
				sDisp_OLED_wr_ypos = 0;
			}
			sDisp_GLCD_wr_ypos += 16;
			if (63 < sDisp_GLCD_wr_ypos){
				sDisp_GLCD_wr_ypos = 0;
			}
		}
	}

}

