//*****************************************
// Graphic OLED & RTC(RX8900) Test Program ATMega328
// CPU Clock:External 8MHz
//   iq[Yf[^@WȂŁAOUWlgɂj
//
//@LCD 128x48 Control IC: ST7565
//@kbc͏ĤPQWSWhbgOtBbNW[gp
//    AQM1248A-RN
//
// File Name:glcd_7565.c
//
//@ȉkbcɕKvȐMꗗ
//
//  PB2: LCD-CS*
//  PB0: LCD-RS
//  PB5: LCD-SCLK
//  PB3: LCD-SDA
//  PB1: LCD-RST*
//
//  C:\WinAVR-20100110\doc\avr-libc\avr-libc-user-manual.pdf ɊeCN[ht@C̐
//
//  2017/05/30  New(TAB4)  Taishi
//  2017/06/26  Modify ohb̃vOC by Taishi
//  2024/10/08  Modify  for mega328_iic_oled_rtc Taishi
//*****************************************
#include	<avr/interrupt.h>
#include	<avr/io.h>
#include	<avr/pgmspace.h>


#include	"timer_h.h"
#include	"pfont5x7_h.h"
#include	"glcd_7565_h.h"

#define		UP_SIDE_DOWN			// |iPVoRlN^j̎BiPVoRlN^j̎̓RgAEg

unsigned char	c_data[3][15];
unsigned char	disp_moji[43];															// kbc\

// ******** MACRO ********
#define	rst_on()				(PORTB &= ~(1 << PB1))						// Reset* Sig:Low
#define	rst_off()				(PORTB |=  (1 << PB1))						// Reset* Sig:High
#define	select_instruction()	(PORTB &= ~(1 << PB0))						// D/I* Sig:Low
#define	select_data()			(PORTB |=  (1 << PB0))						// D/I* Sig:High
#define	cs_on()					(PORTB &= ~(1 << PB2))						// CS Sig:Low
#define	cs_off()				(PORTB |=  (1 << PB2))						// CS Sig:Hogh
#define	sclk_h()				(PORTB |=  (1 << PB5))						// SCLK:High
#define	sclk_l()				(PORTB &= ~(1 << PB5))						// SCLK:Low
#define	sda_h()					(PORTB |=  (1 << PB3))						// SDA:High
#define	sda_l()					(PORTB &= ~(1 << PB3))						// SDA:Low

// ************************************* Graphic LCD Accese *********************
static	unsigned char	c_data0[10];


//******************************
// Get Character Bit Data
// 5x7 Font
//  Get Data Save to c_data0[5];
//******************************
static void	get_asc_data(unsigned char uc)
{
	unsigned short	us;

	us = uc;

	c_data0[0] = pgm_read_byte(&(font5x7[us * 5 + 0]));
	c_data0[1] = pgm_read_byte(&(font5x7[us * 5 + 1]));
	c_data0[2] = pgm_read_byte(&(font5x7[us * 5 + 2]));
	c_data0[3] = pgm_read_byte(&(font5x7[us * 5 + 3]));
	c_data0[4] = pgm_read_byte(&(font5x7[us * 5 + 4]));
}

//******************************
// LCD Bit Data Output
//  uc : Output Data(D0 -- D7)
//
//  \ qrAbr̓ZbgKvL
// *****************************
static void	lcd_G_bitdata_out(unsigned char uc)
{
	unsigned char	i;

	for(i=0x80;i != 0;i >>= 1){				// 8Bit Output
		if((uc & i)==0){					// Out Bit:0(ZERO)
			sda_l();						// SDA:Low
		}else{
			sda_h();						// SDA:High
		}
		sclk_l();							// SCLK:Low
		sclk_h();							// SCLK:High
	}
}
//******************************
// LCD Data Output
//  uc : Output Data(D0 -- D7)
//  rs : RS bit Data(0=Command, 1=Data)
//
// *****************************
static void	lcd_G_one_out(unsigned char uc, int rs)
{
	cs_on();								// CS:ON

	if(rs == 0){							// RS:0(Command)
		select_instruction();				// D/I* Sig:Low
	}else{									// RS:1(Data)
		select_data();						// D/I* Sig:High
	}

	lcd_G_bitdata_out(uc);					// 8Bit Output

	cs_off();								// CS:OFF
}
//******************************
// LCD Command Output
//  uc : Command Data(D0 -- D7)
// *****************************
static	void	lcd_G_cmd_out(unsigned char uc)
{
	lcd_G_one_out(uc, 0);
}
//******************************
// LCD Data Output
//  uc : Data(D0 -- D7)
// *****************************
static	void	lcd_G_data_out(unsigned char uc)
{
	lcd_G_one_out(uc, 1);
}
//******************************
// Init. LCD
//
//  PB2: LCD-CS*
//  PB0: LCD-RS
//  PB5: LCD-SCLK
//  PB3: LCD-SDA
//  PB1: LCD-RST*
//
// *****************************
void	init_lcd_G(void)
{
	cs_off();																	// CS:OFF
	sclk_h();																	// SCLK:High
	DDRB |= ((1 << DDB0)|(1 << DDB1)|(1 << DDB2)|(1 << DDB3)|(1 << DDB5));		// Port-B Bit0--3 & Bit5 : Output

	rst_on();																	// Reset* Sig:Low
	delay_ms(100);																// 1mS
	rst_off();																	// Reset* Sig:High

	delay_ms(100);

	lcd_G_cmd_out(0xAE);														// Display OFF

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	lcd_G_cmd_out(0xA1);
	lcd_G_cmd_out(0xC0);
//	lcd_G_cmd_out(0x10);			sĂω						// Column Address Set Command + A7--A4
//	lcd_G_cmd_out(0x0f);			sĂω						// Column Address Set A3--A0
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
	lcd_G_cmd_out(0xA0);														// ADC Normal
	lcd_G_cmd_out(0xC8);														// Common Output:revers
#endif	//UP_SIDE_DOWN
// ---------------------------------
//	lcd_G_cmd_out(0xA0);														// ADC Normal
//	lcd_G_cmd_out(0xC8);														// Common Output:revers

	lcd_G_cmd_out(0xA3);														// bias:1/7

	// Internal Regulator ON
	lcd_G_cmd_out(0x2C);														// Power Control 1
	delay_ms(10);
	lcd_G_cmd_out(0x2E);														// Power Control 2
	delay_ms(10);
	lcd_G_cmd_out(0x2F);														// Power Control 3
	delay_ms(10);

	// Contrast
	lcd_G_cmd_out(0x23);														// Vo Voltage Resistor Ratio set
	lcd_G_cmd_out(0x81);														// Electrpnic Volume Mode set
//	lcd_G_cmd_out(0x1C);														// Electrpnic Volume Value set
	lcd_G_cmd_out(0x18);														// Electrpnic Volume Value set Modify  for mega328_iic_oled_rtc

	// Display
	lcd_G_cmd_out(0xA4);														// Display All Ponts:normal
	lcd_G_cmd_out(0x40);														// Display Start Line:0(ZERO)
	lcd_G_cmd_out(0xA6);														// Display Nomal/Revers:nomal
	lcd_G_cmd_out(0xAF);														// Display ON

}
//******************************
//  Set Display Offset
//  offset : 0 -- 63(0x3F)
//******************************
void	set_offse_lcd_G(unsigned char offset)
{
	offset &= 0x3f;
	offset |= 0x40;

	lcd_G_cmd_out(offset);	// Display start line set  Value of Offset

}
//******************************
// Clear Display
// *****************************
void	disp_clear_lcd_G(void)
{
	unsigned char	x;
	unsigned char	y;


	for(y=0;y < 8;y++){
		lcd_G_cmd_out((0xb0 | y));	// Page Set
		lcd_G_cmd_out(0x10);		// Column High Address Set

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	lcd_G_cmd_out(0x04);		// Column Low Address Set
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
	lcd_G_cmd_out(0x00);		// Column Low Address Set
#endif	//UP_SIDE_DOWN
// ---------------------------------
//		lcd_G_cmd_out(0x00);		// Column Low Address Set

		for(x=0;x <128; x++){
			lcd_G_data_out(0x00);
		}
	}
}
//******************************
// 8 Line Clear Display
//  y : Position Y(0 -- 63)
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_clear_8line_lcd_G(unsigned char y)
{
	unsigned char	x;

	y &= 0x3f;
	y /= 8;

	lcd_G_cmd_out((0xb0 | y));	// Page Set
	lcd_G_cmd_out(0x10);		// Column High Address Set

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	lcd_G_cmd_out(0x04);		// Column Low Address Set
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
	lcd_G_cmd_out(0x00);		// Column Low Address Set
#endif	//UP_SIDE_DOWN
// ---------------------------------

	for(x=0;x <128; x++){
		lcd_G_data_out(0x00);
	}
}
//******************************
// display LCD One charcter
//  pos_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63)
//  uc : Charcter
//    E\sb`͂UhbgitHgԂPhbgLj
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_one_char_lcd_G_6x8(unsigned char pos_x, unsigned char pos_y, unsigned char uc)
{
	pos_x &= 0x7f;
	pos_y &= 0x3f;

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	pos_y /= 8;
	get_asc_data(uc);						// Get Character Bit Data
	lcd_G_cmd_out((0xb0 | pos_y));			// Page Set
	lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
	lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
	lcd_G_data_out(c_data0[0]);
	lcd_G_data_out(c_data0[1]);
	lcd_G_data_out(c_data0[2]);
	lcd_G_data_out(c_data0[3]);
	lcd_G_data_out(c_data0[4]);
	lcd_G_data_out(0x00);
}
//******************************
// display LCD One charcter
//  pos_x : Position X(0 -- 20)
//  pos_y : Position Y(0 -- 7)
//  uc : Charcter
//    E\sb`͂UhbgitHgԂPhbgLj
// *****************************
void	disp_one_char_lcd_G(unsigned char pos_x, unsigned char pos_y, unsigned char uc)
{
	pos_x *= 6;

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	get_asc_data(uc);						// Get Character Bit Data
	lcd_G_cmd_out((0xb0 | pos_y));			// Page Set
	lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
	lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
	lcd_G_data_out(c_data0[0]);
	lcd_G_data_out(c_data0[1]);
	lcd_G_data_out(c_data0[2]);
	lcd_G_data_out(c_data0[3]);
	lcd_G_data_out(c_data0[4]);
	lcd_G_data_out(0x00);
}
//******************************
// display LCD One charcteri]\j
//  pos_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63)
//  uc : Charcter
//    E\sb`͂UhbgitHgԂPhbgLj
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_one_char_lcd_G_6x8_reverse(unsigned char pos_x, unsigned char pos_y, unsigned char uc)
{
	pos_x &= 0x7f;
	pos_y &= 0x3f;

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	pos_y /= 8;
	get_asc_data(uc);						// Get Character Bit Data
	lcd_G_cmd_out((0xb0 | pos_y));			// Page Set
	lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
	lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
	lcd_G_data_out(c_data0[0] ^ 0xff);
	lcd_G_data_out(c_data0[1] ^ 0xff);
	lcd_G_data_out(c_data0[2] ^ 0xff);
	lcd_G_data_out(c_data0[3] ^ 0xff);
	lcd_G_data_out(c_data0[4] ^ 0xff);
	lcd_G_data_out(0xff);
}
//******************************
// display LCD One charcteri]\j
//  pos_x : Position X(0 -- 20)
//  pos_y : Position Y(0 -- 7)
//  uc : Charcter
//    E\sb`͂UhbgitHgԂPhbgLj
// *****************************
void	disp_one_char_lcd_G_reverse(unsigned char pos_x, unsigned char pos_y, unsigned char uc)
{
	pos_x *= 6;

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	get_asc_data(uc);						// Get Character Bit Data
	lcd_G_cmd_out((0xb0 | pos_y));			// Page Set
	lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
	lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
	lcd_G_data_out(c_data0[0] ^ 0xff);
	lcd_G_data_out(c_data0[1] ^ 0xff);
	lcd_G_data_out(c_data0[2] ^ 0xff);
	lcd_G_data_out(c_data0[3] ^ 0xff);
	lcd_G_data_out(c_data0[4] ^ 0xff);
	lcd_G_data_out(0xff);
}
//******************************
// display LCD One charcter 24x24 Font
//  pos_x : Position X(-24 -- 127)
//  pos_y : Position Y(0 -- 63)
//  puc : Pointer of Charcter
//
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
//  tHǧ` [24][3] 
//    [A][B] ̂` [0][B]ō [23][B]ŉEɂȂ
//    [A][B] ̂ac [A][2]㑤 [A][0]ɂȂ
// *****************************
void	disp_one_char_lcd_G_24x24(short pos_x, short pos_y, unsigned char *puc)
{
	short	i;
	short	loop;
	short	start;

	start = 0;
	loop = 24;
	if(pos_x < 0){
		loop += pos_x;
		start = 0 - pos_x;
		pos_x = 0;
	}

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	if(0 < loop){
		// Upside
		lcd_G_cmd_out(0xb0 | (unsigned char)(pos_y / 8));		// Page Set
		lcd_G_cmd_out(0x10 | (((unsigned char)pos_x) >> 4));	// Column High Address Set
		lcd_G_cmd_out((unsigned char)(pos_x & 0x0f));			// Column Low Address Set
		for(i=start;i<24;i++){
			if(127 < (pos_x + i)){
				break;
			}
			lcd_G_data_out(*(puc+i*3+2));
		}

		// Middle
		lcd_G_cmd_out(0xb0 | (unsigned char)((pos_y / 8)+1));	// Page Set
		lcd_G_cmd_out(0x10 | (((unsigned char)pos_x) >> 4));	// Column High Address Set
		lcd_G_cmd_out((unsigned char)(pos_x & 0x0f));			// Column Low Address Set
		for(i=start;i<24;i++){
			if(127 < (pos_x + i)){
				break;
			}
			lcd_G_data_out(*(puc+i*3+1));
		}

		//Downside
		lcd_G_cmd_out(0xb0 | (unsigned char)((pos_y / 8)+2));	// Page Set
		lcd_G_cmd_out(0x10 | ((unsigned char)(pos_x) >> 4));	// Column High Address Set
		lcd_G_cmd_out((unsigned char)(pos_x & 0x0f));			// Column Low Address Set
		for(i=start;i<24;i++){
			if(127 < (pos_x + i)){
				break;
			}
			lcd_G_data_out(*(puc+i*3+0));
		}
	}

}
//******************************
// display LCD 24x24 Font Area Clear
//  pos_sx : Start Position X(0 -- 127)
//  pos_ex : End Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63)
//  rev : Reverse(0,1)
//
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_clear_lcd_G_24x24(short pos_sx, short pos_ex, short pos_y, unsigned char rev)
{
	short	i;
	unsigned char	spdata = 0;

	if(rev != 0){
		spdata = 0xff;
	}

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_sx += 4;
	pos_ex += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	// Upside
	lcd_G_cmd_out(0xb0 | (unsigned char)(pos_y / 8));		// Page Set
	lcd_G_cmd_out(0x10 | (((unsigned char)pos_sx) >> 4));	// Column High Address Set
	lcd_G_cmd_out((unsigned char)(pos_sx & 0x0f));			// Column Low Address Set
	for(i=pos_sx;i<=pos_ex;i++){
		if(127 < i){
			break;
		}
		lcd_G_data_out(spdata);
	}

	//Middle
	lcd_G_cmd_out(0xb0 | (unsigned char)((pos_y / 8)+1));	// Page Set
	lcd_G_cmd_out(0x10 | ((unsigned char)(pos_sx) >> 4));	// Column High Address Set
	lcd_G_cmd_out((unsigned char)(pos_sx & 0x0f));			// Column Low Address Set
	for(i=pos_sx;i<=pos_ex;i++){
		if(127 < (pos_sx + i)){
			break;
		}
		lcd_G_data_out(spdata);
	}

	//Downside
	lcd_G_cmd_out(0xb0 | (unsigned char)((pos_y / 8)+2));	// Page Set
	lcd_G_cmd_out(0x10 | ((unsigned char)(pos_sx) >> 4));	// Column High Address Set
	lcd_G_cmd_out((unsigned char)(pos_sx & 0x0f));			// Column Low Address Set
	for(i=pos_sx;i<=pos_ex;i++){
		if(127 < (pos_sx + i)){
			break;
		}
		lcd_G_data_out(spdata);
	}

}

//******************************
// display LCD tate line
//  pos_x : Position X(0 -- 127)
//  str_y : Position Y(0 -- 63)
//  end_y : Position Y(0 -- 63)
//
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
//  xʒu̓hbg̎wɂȂĂ܂B
//    W̔{[PŖ͍܂ł̃hbg鎖L܂
// *****************************
const	unsigned char str_bit_data[8]={
		0b11111111,				// D0
		0b11111110,				// D0 -- D1
		0b11111100,				// D0 -- D2
		0b11111000,				// D0 -- D3
		0b11110000,				// D0 -- D4
		0b11100000,				// D0 -- D5
		0b11000000,				// D0 -- D6
		0b10000000,				// D0 -- D7
};

const	unsigned char end_bit_data[8]={
		0b00000001,				// D0
		0b00000011,				// D0 -- D1
		0b00000111,				// D0 -- D2
		0b00001111,				// D0 -- D3
		0b00011111,				// D0 -- D4
		0b00111111,				// D0 -- D5
		0b01111111,				// D0 -- D6
		0b11111111,				// D0 -- D7
};

void	disp_tate_line_lcd_G(unsigned char pos_x, unsigned char str_y, unsigned char end_y)
{
	unsigned char	uc;
	unsigned char	i;
	unsigned char	amari_str;
	unsigned char	amari_end;
	unsigned char	str;
	unsigned char	end;
	
// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	str = (str_y / 8);
	end = (end_y / 8);
	if(str == end){								// WrbgŏÎłPf[^o
		amari_str = ((str_y) % 8);
		amari_end = ((end_y) % 8);
		uc = str_bit_data[amari_str];
		uc &= end_bit_data[amari_end];
		lcd_G_cmd_out(0xb0 | str);				// Page Set
		lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
		lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
		lcd_G_data_out(uc);
	}else{
		amari_str = ((str_y) % 8);
		amari_end = ((end_y) % 8);
		uc = str_bit_data[amari_str];
		lcd_G_cmd_out(0xb0 | str);				// Page Set
		lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
		lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
		lcd_G_data_out(uc);
		for(i=str;i<end;i++){
			lcd_G_cmd_out(0xb0 | i);			// Page Set
			lcd_G_cmd_out(0x10 | (pos_x >> 4));	// Column High Address Set
			lcd_G_cmd_out(pos_x & 0x0f);		// Column Low Address Set
			lcd_G_data_out(0xff);
		}
		uc = end_bit_data[amari_end];
		lcd_G_cmd_out(0xb0 | end);				// Page Set
		lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
		lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
		lcd_G_data_out(uc);
	}

}
//******************************
// display LCD yoko line(1dot)
//  str_x : Position X(0 -- 127)
//  end_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63)
//
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
//    ㉺̃hbg܂
// *****************************
const	unsigned char center_bit_data[8]={
		0b00000001,				// D0
		0b00000010,				// D1
		0b00000100,				// D2
		0b00001000,				// D3
		0b00010000,				// D4
		0b00100000,				// D5
		0b01000000,				// D6
		0b10000000,				// D7
};
void	disp_yoko_line_lcd_G(unsigned char str_x, unsigned char end_x, unsigned char pos_y)
{
	unsigned char	uc;
	unsigned char	i;
	unsigned char	amari;

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	str_x += 4;
	end_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	amari = ((pos_y) % 8);
	uc = center_bit_data[amari];
	lcd_G_cmd_out(0xb0 | (pos_y / 8));		// Page Set
	lcd_G_cmd_out(0x10 | (str_x >> 4));		// Column High Address Set
	lcd_G_cmd_out(str_x & 0x0f);			// Column Low Address Set
	for(i=str_x; i < end_x; i++){
		lcd_G_data_out(uc);
	}
}
//******************************
// display LCD(20char x 8Line)
//  pos_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63,\̂ 0 -- 47)
//  ucp : String(termnate 0x00)
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_lcd_G_6x8(unsigned char pos_x, unsigned char pos_y, unsigned char *ucp)
{
	unsigned char	pos;

	pos_y &= 0x3f;
	pos = (pos_x & 0x7f);
	while(*ucp != 0x00){				// String Data Out
		if(127 < pos){
			break;
		}
		disp_one_char_lcd_G_6x8(pos, pos_y, *ucp);
		ucp++;
		pos += 6;
	}
}
//******************************
// display LCD(20char x 8Line, ]\)
//  pos_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63,\̂ 0 -- 47)
//  ucp : String(termnate 0x00)
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_lcd_G_6x8_reverse(unsigned char pos_x, unsigned char pos_y, unsigned char *ucp)
{
	unsigned char	pos;

	pos_y &= 0x3f;
	pos = (pos_x & 0x7f);
	while(*ucp != 0x00){				// String Data Out
		if(127 < pos){
			break;
		}
		disp_one_char_lcd_G_6x8_reverse(pos, pos_y, *ucp);
		ucp++;
		pos += 6;
	}
}

// TVtHgPOPUɊĝׂQhbg̃f[^
const	unsigned char glbt_07_10x16[8][2]={						// 5x7  10x16 ɑ傫鎞Ɏg
									{0b00000011, 0b00000000},	// D0
									{0b00001100, 0b00000000},	// D1
									{0b00110000, 0b00000000},	// D2
									{0b11000000, 0b00000000},	// D3
									{0b00000000, 0b00000011},	// D4
									{0b00000000, 0b00001100},	// D5
									{0b00000000, 0b00110000},	// D6
									{0b00000000, 0b11000000},	// D7
								};
//******************************
// Get Character Bit Data
// 10x16 Font
//  Get Data Save to c_data[3][15];
//******************************
static void	get_asc_data_10x16(unsigned char uc)
{
	unsigned short	us;
	unsigned char	ucd;
	unsigned char	i;
	unsigned char	j;
	unsigned char	k;
	unsigned char	bt;

	us = uc;
	us *= 5;

	for(i=0;i<10;i++){										// tHg鏊NA
		for(j=0;j<2;j++){
			c_data[j][i] = 0;
		}
	}

	for(i=0;i<5;i++){										// 5x7 tHg擾郋[v
		ucd = pgm_read_byte(&(font5x7[us + i]));
		for(bt=0x01,k=0;k<8;k++, bt<<= 1){					// 擾Wrbg̃f[^郋[v
			if((ucd & bt)!=0){
				for(j=0;j<2;j++){							// 5x7 ̂Prbg̃f[^2x2 ̃f[^ɂ郋[vi͂PỎj
					c_data[0][i*2+j] |= glbt_07_10x16[k][0];
					c_data[1][i*2+j] |= glbt_07_10x16[k][1];
				}
			}
		}
	}

}
//******************************
// display LCD One charcter 12x16 Font
//  pos_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63)
//  uc : Charcter
//    E\sb`͂PQbgitHgԂQhbgLj
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_one_char_lcd_G_12x16(unsigned char pos_x, unsigned char pos_y, unsigned char uc)
{
	unsigned char	i;

	pos_x &= 0x7f;

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	pos_y &= 0x3f;
	pos_y /= 8;

	get_asc_data_10x16(uc);						// Get Character Bit Data

	for(i=0;i<2;i++){
		lcd_G_cmd_out(0xb0 | (pos_y+i));		// Page Set
		lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
		lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
		lcd_G_data_out(c_data[i][0]);
		lcd_G_data_out(c_data[i][1]);
		lcd_G_data_out(c_data[i][2]);
		lcd_G_data_out(c_data[i][3]);
		lcd_G_data_out(c_data[i][4]);
		lcd_G_data_out(c_data[i][5]);
		lcd_G_data_out(c_data[i][6]);
		lcd_G_data_out(c_data[i][7]);
		lcd_G_data_out(c_data[i][8]);
		lcd_G_data_out(c_data[i][9]);
		lcd_G_data_out(0x00);
		lcd_G_data_out(0x00);
	}
}
//******************************
// display LCD One charcter 12x16 Font Reverse
//  pos_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63)
//  uc : Charcter
//    E\sb`͂PQbgitHgԂQhbgLj
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_one_char_lcd_G_12x16_reverse(unsigned char pos_x, unsigned char pos_y, unsigned char uc)
{
	unsigned char	i;

	pos_x &= 0x7f;

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	pos_y &= 0x3f;
	pos_y /= 8;

	get_asc_data_10x16(uc);						// Get Character Bit Data

	for(i=0;i<2;i++){
		lcd_G_cmd_out(0xb0 | (pos_y+i));		// Page Set
		lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
		lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
		lcd_G_data_out(c_data[i][0] ^ 0xff);
		lcd_G_data_out(c_data[i][1] ^ 0xff);
		lcd_G_data_out(c_data[i][2] ^ 0xff);
		lcd_G_data_out(c_data[i][3] ^ 0xff);
		lcd_G_data_out(c_data[i][4] ^ 0xff);
		lcd_G_data_out(c_data[i][5] ^ 0xff);
		lcd_G_data_out(c_data[i][6] ^ 0xff);
		lcd_G_data_out(c_data[i][7] ^ 0xff);
		lcd_G_data_out(c_data[i][8] ^ 0xff);
		lcd_G_data_out(c_data[i][9] ^ 0xff);
		lcd_G_data_out(0xff);
		lcd_G_data_out(0xff);
	}
}
//******************************
// display LCD 12x16 Font (10char x 3Line)
//  pos_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63)
//  ucp : String(termnate 0x00)
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_lcd_G_12x16(unsigned char pos_x, unsigned char pos_y, unsigned char *ucp)
{
	unsigned char	pos;

	pos = pos_x;
	while(*ucp != 0x00){				// String Data Out
		disp_one_char_lcd_G_12x16(pos, pos_y, *ucp);
		ucp++;
		pos += 12;
	}
}

//******************************
// display LCD 12x16 Font (10char x 3Line)
//  pos_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63)
//  ucp : String(termnate 0x00)
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_lcd_G_12x16_reverse(unsigned char pos_x, unsigned char pos_y, unsigned char *ucp)
{
	unsigned char	pos;

	pos = pos_x;
	while(*ucp != 0x00){				// String Data Out
		disp_one_char_lcd_G_12x16_reverse(pos, pos_y, *ucp);
		ucp++;
		pos += 12;
	}
}

//******************************
// Clear 16line LCD
//  pos_y : Position Y(0 -- 63)
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_clear_16line_lcd_G(unsigned char pos_y)
{
	unsigned char	pos_x=0;
	unsigned char	y;
	unsigned char	x;

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	pos_y &= 0x3f;
	pos_y /= 8;

	for(y=0;y<2;y++){
		if(7 < (pos_y + y)){
			break;
		}
		lcd_G_cmd_out(0xb0 | (pos_y+y));		// Page Set
		lcd_G_cmd_out(0x10);					// Column High Address Set
		lcd_G_cmd_out(pos_x);					// Column Low Address Set
		for(x=0;x <128; x++){
			lcd_G_data_out(0x00);
		}
	}
}


// TVtHgPTQSɊĝׂRhbg̃f[^
const	unsigned char glbt_07_15x24[8][3]={											// 5x7  15x24 ɑ傫鎞Ɏg
									{0b00000111, 0b00000000, 0b00000000},	// D0
									{0b00111000, 0b00000000, 0b00000000},	// D1
									{0b11000000, 0b00000001, 0b00000000},	// D2
									{0b00000000, 0b00001110, 0b00000000},	// D3
									{0b00000000, 0b01110000, 0b00000000},	// D4
									{0b00000000, 0b10000000, 0b00000011},	// D5
									{0b00000000, 0b00000000, 0b00011100},	// D6
									{0b00000000, 0b00000000, 0b11100000},	// D7
								};
//******************************
// Get Character Bit Data
// 15x24 Font
//  Get Data Save to c_data[3][15];
//******************************
static void	get_asc_data_15x24(unsigned char uc)
{
	unsigned short	us;
	unsigned char	ucd;
	unsigned char	i;
	unsigned char	j;
	unsigned char	k;
	unsigned char	bt;

	us = uc;
	us *= 5;

	for(i=0;i<15;i++){										// tHg鏊NA
		for(j=0;j<3;j++){
			c_data[j][i] = 0;
		}
	}

	for(i=0;i<5;i++){										// 5x7 tHg擾郋[v
		ucd = pgm_read_byte(&(font5x7[us + i]));
		for(bt=0x01,k=0;k<8;k++, bt<<= 1){					// 擾Wrbg̃f[^郋[v
			if((ucd & bt)!=0){
				for(j=0;j<3;j++){							// 5x7 ̂Prbg̃f[^3x3 ̃f[^ɂ郋[vi͂PT̉j
					c_data[0][i*3+j] |= glbt_07_15x24[k][0];
					c_data[1][i*3+j] |= glbt_07_15x24[k][1];
					c_data[2][i*3+j] |= glbt_07_15x24[k][2];
				}
			}
		}
	}

}
//******************************
// display LCD One charcter 18x24 Font
//  pos_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63)
//  uc : Charcter
//    E\sb`͂PWhbgitHgԂRhbgLj
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_one_char_lcd_G_18x24(unsigned char pos_x, unsigned char pos_y, unsigned char uc)
{
	unsigned char	i;

	pos_x &= 0x7f;

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	pos_y &= 0x3f;
	pos_y /= 8;

	get_asc_data_15x24(uc);						// Get Character Bit Data

	for(i=0;i<3;i++){
		lcd_G_cmd_out(0xb0 | (pos_y+i));		// Page Set
		lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
		lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
		lcd_G_data_out(c_data[i][0]);
		lcd_G_data_out(c_data[i][1]);
		lcd_G_data_out(c_data[i][2]);
		lcd_G_data_out(c_data[i][3]);
		lcd_G_data_out(c_data[i][4]);
		lcd_G_data_out(c_data[i][5]);
		lcd_G_data_out(c_data[i][6]);
		lcd_G_data_out(c_data[i][7]);
		lcd_G_data_out(c_data[i][8]);
		lcd_G_data_out(c_data[i][9]);
		lcd_G_data_out(c_data[i][10]);
		lcd_G_data_out(c_data[i][11]);
		lcd_G_data_out(c_data[i][12]);
		lcd_G_data_out(c_data[i][13]);
		lcd_G_data_out(c_data[i][14]);
		lcd_G_data_out(0x00);
		lcd_G_data_out(0x00);
		lcd_G_data_out(0x00);
	}
}
//******************************
// display LCD One charcter 18x24 Font reversw
//  pos_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63)
//  uc : Charcter
//    E\sb`͂PWhbgitHgԂRhbgLj
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_one_char_lcd_G_18x24_reverse(unsigned char pos_x, unsigned char pos_y, unsigned char uc)
{
	unsigned char	i;

	pos_x &= 0x7f;

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	pos_y &= 0x3f;
	pos_y /= 8;

	get_asc_data_15x24(uc);						// Get Character Bit Data

	for(i=0;i<3;i++){
		lcd_G_cmd_out(0xb0 | (pos_y+i));		// Page Set
		lcd_G_cmd_out(0x10 | (pos_x >> 4));		// Column High Address Set
		lcd_G_cmd_out(pos_x & 0x0f);			// Column Low Address Set
		lcd_G_data_out(c_data[i][0] ^ 0xff);
		lcd_G_data_out(c_data[i][1] ^ 0xff);
		lcd_G_data_out(c_data[i][2] ^ 0xff);
		lcd_G_data_out(c_data[i][3] ^ 0xff);
		lcd_G_data_out(c_data[i][4] ^ 0xff);
		lcd_G_data_out(c_data[i][5] ^ 0xff);
		lcd_G_data_out(c_data[i][6] ^ 0xff);
		lcd_G_data_out(c_data[i][7] ^ 0xff);
		lcd_G_data_out(c_data[i][8] ^ 0xff);
		lcd_G_data_out(c_data[i][9] ^ 0xff);
		lcd_G_data_out(c_data[i][10] ^ 0xff);
		lcd_G_data_out(c_data[i][11] ^ 0xff);
		lcd_G_data_out(c_data[i][12] ^ 0xff);
		lcd_G_data_out(c_data[i][13] ^ 0xff);
		lcd_G_data_out(c_data[i][14] ^ 0xff);
		lcd_G_data_out(0xff);
		lcd_G_data_out(0xff);
		lcd_G_data_out(0xff);
	}
}
//******************************
// display LCD 18x24 Font (8char x 2Line)
//  pos_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63)
//  ucp : String(termnate 0x00)
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_lcd_G_18x24(unsigned char pos_x, unsigned char pos_y, unsigned char *ucp)
{
	unsigned char	pos;

	pos = pos_x;
	while(*ucp != 0x00){				// String Data Out
		disp_one_char_lcd_G_18x24(pos, pos_y, *ucp);
		ucp++;
		pos += 18;
	}
}

//******************************
// display LCD 18x24 Font (8char x 2Line)
//  pos_x : Position X(0 -- 127)
//  pos_y : Position Y(0 -- 63)
//  ucp : String(termnate 0x00)
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_lcd_G_18x24_reverse(unsigned char pos_x, unsigned char pos_y, unsigned char *ucp)
{
	unsigned char	pos;

	pos = pos_x;
	while(*ucp != 0x00){				// String Data Out
		disp_one_char_lcd_G_18x24_reverse(pos, pos_y, *ucp);
		ucp++;
		pos += 18;
	}
}

//******************************
// Clear 24line LCD
//  pos_y : Position Y(0 -- 63)
//  xʒu̓hbg̎wɂȂĂ܂Aۂ͂WhbgɂȂ܂B
//    0 -> 7 ͂OA8 -> 15 ͂WA16 -> 23͂PUA24 -> 29͂QSA30 -> 35͂ROA36 -> 41͂RUA42 -> 47͂SQ ɂȂ܂B
// *****************************
void	disp_clear_24line_lcd_G(unsigned char pos_y)
{
	unsigned char	pos_x = 0;
	unsigned char	y;
	unsigned char	x;

// ---------------------------------
#ifdef	UP_SIDE_DOWN
// \㉺t̎͂炪
	pos_x += 4;
#else	//UP_SIDE_DOWN
// \ʂ̎͂炪
#endif	//UP_SIDE_DOWN
// ---------------------------------

	pos_y &= 0x3f;
	pos_y /= 8;

	for(y=0;y<3;y++){
		lcd_G_cmd_out(0xb0 | (pos_y+y));		// Page Set
		lcd_G_cmd_out(0x10);					// Column High Address Set
		lcd_G_cmd_out(pos_x);					// Column Low Address Set
		for(x=0;x <128; x++){
			lcd_G_data_out(0x00);
		}
	}
}

