//*****************************************
// Graphic OLED & RTC(RX8900) Test Program ATMega328
// CPU Clock:External 8MHz
//   iq[Yf[^@WȂŁAOUWlgɂj
//
//
//@OLED 128x64 Control IC: SSD1306
//    H@nkdc\ihhbj
//
//@RTC RX8900(EPSON)
//
// File Name:mega328_iic_oled_rtc.c
//
//  PC5(SCL): TWI-SCL
//  PC4(SDA): TWI-SDA
//
//  C:\WinAVR-20100110\doc\avr-libc\avr-libc-user-manual.pdf ɊeCN[ht@C̐
//
//  2023/03/07 New  for mega328_i2c_oled Taishi
//  2023/05/13 Modify  for mega328_iic_oled_rtc Taishi
//
//*****************************************
#include	<avr/interrupt.h>
#include	<avr/io.h>


#include	"timer_h.h"
#include	"mega_Twi_h.h"
#include	"port_bit_h.h"
#include	"oled_1306_h.h"
#include	"disp_clock_data_h.h"
#include	"Rtc_Rx8900_h.h"
#include	"set_rtc_h.h"
#include	"ExtIO_Pca9536_h.h"
#include	"glcd_7565_h.h"
#include	"ir_dsm_38khz_h.h"
#include	"pr_ir_nixie_mode_h.h"
#include	"pr_ir_nec_mode_h.h"
#include	"disp_ir_nixie_data_h.h"
#include	"pr_clock_mode_h.h"
#include	"pr_gps_mode_h.h"
#include	"eep_dat_h.h"

#include	"mega328_iic_oled_rtc_h.h"


enum DISP_FUNC_ITEM {		// \֐
		F_6x8 = 0,			// UW\
		F_12x16,			// PQPU\
		F_18x24,			// PWQS\
		F_24x32,			// QSRQ\

		F_END
	};

struct DISP_SWITCH_CONST {
	unsigned char run_mode;		//]\郂[h
	unsigned char dc_fsize;		//\tHgTCY
	unsigned char dc_x;			//\ʒuw
	unsigned char dc_y;			//\ʒux
	         char *dc_pdt;		//

};
struct DISP_SELECT_CONST {
	unsigned char dc_fsize;		//\tHgTCY
	unsigned char dc_x;			//\ʒuw
	unsigned char dc_y;			//\ʒux
	         char *dc_pdt;		//

};
static
struct DISP_SELECT_CONST	sDisp_select_const[]={
	{F_12x16,   0*6, 0*8, "MODE"},
	{F_12x16,   9*6, 0*8, "SELECT"},

	{F_END,     0,   0,   ""},			// END I
};

static
struct DISP_SWITCH_CONST	sDisp_switch_const[]={
	{RUN_CLOCK,    F_12x16,   0*6, 2*8, "CLOCK"},		// 'CLOCK'
	{RUN_IR_NIXIE, F_12x16,  11*6, 2*8, "NIXIE"},		// 'IR NIXIE'
	{RUN_IR_NEC,   F_12x16,   0*6, 4*8, "NEC"},			// 'IR NEC'
	{RUN_GPS,      F_12x16,  11*6, 4*8, "GPS"},			// 'GPS'

	{RUN_END,      F_12x16,     0,   0,   ""},			// END I
};


unsigned char	test_string[20] = {'1','2','3','4','5','6','7','8','9','0','A',0x00};

unsigned char	gRun_mode = RUN_CLOCK;

//*****************************************
// Iʂ̃^CǧŒ\
//*****************************************
void	disp_select_mode_const(void)
{
	unsigned char		bn;
	unsigned char		*pbf;
	unsigned char		pos_x;				// hbgPʂ̂wʒu
	unsigned char		pos_y;				// hbgPʂ̂xʒu
	unsigned char		u_func;				// gp֐

	for(bn = 0;  ;bn++){
		if(sDisp_select_const[bn].dc_fsize == F_END){	// tHgTCY eQdlc͕̎\ȂŏI
			break;
		}

		u_func = sDisp_select_const[bn].dc_fsize;	// gp֐̎
		pos_x = sDisp_select_const[bn].dc_x;			// pos_x
		pos_y = sDisp_select_const[bn].dc_y;			// pos_y
		pbf = (unsigned char *)(sDisp_select_const[bn].dc_pdt);			// \

		if(u_func == F_18x24){
			disp_oled_G_18x24(pos_x, pos_y, pbf);	// x, y, (18x24)
			disp_lcd_G_18x24(pos_x, pos_y, pbf);
		}else if(u_func == F_12x16){
			disp_oled_G_12x16(pos_x, pos_y, pbf);	// x, y, data(12x16)
			disp_lcd_G_12x16(pos_x, pos_y, pbf);
		}else{
			disp_oled_G_6x8(pos_x, pos_y, pbf);			// x, y, data(6x8)
			disp_lcd_G_6x8( pos_x,  pos_y, pbf);
		}
	}
}
//*****************************************
// Iʂ̃XCb`̌Œ\
//*****************************************
void	disp_switch_const(unsigned char sel)
{
	unsigned char		mode;				// ]w胂[h
	unsigned char		bn;
	unsigned char		*pbf;
	unsigned char		pos_x;				// hbgPʂ̂wʒu
	unsigned char		pos_y;				// hbgPʂ̂xʒu
	unsigned char		u_func;				// gp֐


	for(bn = 0;  ;bn++){

		mode = sDisp_switch_const[bn].run_mode;		// ]w胂[h
		if(mode == RUN_END){						// [h qtmQdlc͕̎\ȂŏI
			break;
		}
		u_func = sDisp_switch_const[bn].dc_fsize;	// gp֐̎
		pos_x = sDisp_switch_const[bn].dc_x;			// pos_x
		pos_y = sDisp_switch_const[bn].dc_y;			// pos_y
		pbf = (unsigned char *)(sDisp_switch_const[bn].dc_pdt);			// \

		if(u_func == F_18x24){
			if(sel != mode){
				disp_oled_G_18x24(pos_x, pos_y, pbf);	// x, y, (18x24)
				disp_lcd_G_18x24(pos_x, pos_y, pbf);
			}else{
				disp_oled_G_18x24_reverse(pos_x, pos_y, pbf);	// x, y, (18x24)
				disp_lcd_G_18x24_reverse(pos_x, pos_y, pbf);
			}
		}else if(u_func == F_12x16){
			if(sel != mode){
				disp_oled_G_12x16(pos_x, pos_y, pbf);	// x, y, data(12x16)
				disp_lcd_G_12x16(pos_x, pos_y, pbf);
			}else{
				disp_oled_G_12x16_reverse(pos_x, pos_y, pbf);	// x, y, data(12x16)
				disp_lcd_G_12x16_reverse(pos_x, pos_y, pbf);
			}
		}else{
			if(sel != mode){
				disp_oled_G_6x8(pos_x, pos_y, pbf);			// x, y, data(6x8)
				disp_lcd_G_6x8(pos_x,  pos_y, pbf);
			}else{
				disp_oled_G_6x8_reverse(pos_x, pos_y, pbf);			// x, y, data(6x8)
				disp_lcd_G_6x8_reverse(pos_x,  pos_y, pbf);
			}
		}
	}
}

//*****************************************
// [hI
//*****************************************
void	select_mode(void)
{
	unsigned char	sel;
	unsigned short	us;

	sel = gRun_mode;

	disp_select_mode_const();	// Iʂ̃^CǧŒ\
	wait_sw_off();
	us = get_action_sw();						// XCb`Ԃ
	delay_ms(50);
	us = get_action_sw();						// XCb`Ԃ
	delay_ms(50);
	us = get_action_sw();						// XCb`Ԃ
	delay_ms(50);
	us = get_action_sw();						// XCb`Ԃ
	delay_ms(50);

	disp_switch_const(sel);		// Iʂ̃XCb`̌Œ\

	while(1){
		us = get_action_sw();						// XCb`Ԃ
		us &= SW_MASK;								// XCb`ɂ
		if(us != 0){								// ANVL鎞
			proc_no_key_timeup(1);					// L[͂L
			if((us & SW1_OFF_LONG) != 0){			// ꎞ(set switch)
				gRun_mode = sel;
				break;
			}else if((us & SW2_OFF_SHORT) != 0){	// Zꎞ(select switch)
				sel++;
				if(sel == RUN_END){
					sel = RUN_CLOCK;
				}
				disp_switch_const(sel);				// Iʂ̃XCb`̌Œ\
			}
		}

	}
}

//*****************************************
// C
//*****************************************
int main(void)
{
	unsigned char	uc;
	unsigned short	us;
	unsigned char	toggle = 0;

	led_red_on_off(1);
	delay_ms(250);

	twi_init();
	ext_io_init();						// Ext. IO PCA9536 Initial

	led_grn_on_off(1);
	init_oled_G();						// Init. OLED
	disp_clear_oled_G();
	delay_ms(250);

	led_blu_on_off(1);
	init_lcd_G();						// Init, GLCD
	disp_clear_lcd_G();
	delay_ms(250);

	led_ext_red_on_off(1);
	timer2_init();						// Timer2 Initial
	sw_init();							// XCb`֌W̏ݒ
	delay_ms(250);
	gRun_mode = read_eep_run_mode();			//  ddoqnl qtmQlncd ǂݍ

	led_ext_blu_on_off(1);
	sei();								// 荞݉
	delay_ms(100);
	us = get_action_sw();						// XCb`Ԃ
	delay_ms(100);
	us = get_action_sw();						// XCb`Ԃ
	delay_ms(100);
	us = get_action_sw();						// XCb`Ԃ

	led_red_on_off(0);
	led_grn_on_off(0);
	led_blu_on_off(0);
	led_ext_red_on_off(0);
	led_ext_blu_on_off(0);
	delay_ms(100);

	led_red_on_off(1);
	led_grn_on_off(1);
	led_blu_on_off(1);

	us = get_action_sw();						// XCb`Ԃ
	us &= SW_MASK;								// XCb`ɂ
	if(us != 0){								// ANVL鎞
		proc_no_key_timeup(1);					// L[͂L
		if((us & (SW1_ON | SW2_ON)) == (SW1_ON | SW2_ON)){	// Q̃XCb`nm(set & select switch)
			uc = gRun_mode;
			select_mode();						// [hI
			if(uc != gRun_mode){
				cli();
				write_eep_run_mode(gRun_mode);
				sei();							// 荞݉
			}
		}
	}


	if(gRun_mode == RUN_IR_NIXIE){
		ir_nixer_mode_main();			// jLV[[h̃C
	}else if(gRun_mode == RUN_IR_NEC){
		ir_nec_mode_main();				// mdb[h̃C
	}else if(gRun_mode == RUN_GPS){
		gps_mode_main();				// for[h̃C
	}else{
		clock_mode_main();				// v[h̃C
	}

	disp_clear_oled_G();				// Clear All Display
	disp_clear_lcd_G();

	led_red_on_off(0);
	led_grn_on_off(0);
	led_blu_on_off(0);

	led_blu_on_off(1);
	uc = rtc_init();								//  qsb̏ݒ
	if(uc != 0){
		rtc_error();								// qsbG[
	}
	delay_ms(100);
	led_blu_on_off(0);

	led_grn_on_off(1);
	uc = rtc_read();								// RTC N ǂݏo
	if(uc != 0){
		rtc_error();								// qsbG[
	}
	uc = rtc_tempe_read();							// RTC xf[^ǂݏo
	if(uc != 0){
		rtc_error();								// qsbG[
	}
	delay_ms(1);
	led_grn_on_off(0);

	timer1_init();
	init_dsm();


	disp_clock_const();								// v̌Œ\
	disp_clock_data(1);								// vf[^̕\i1=Sĕ\j
	while(1){
		us = get_action_sw();						// XCb`Ԃ
		us &= SW_MASK;								// XCb`ɂ
		if(us != 0){								// ANVL鎞
			proc_no_key_timeup(1);					// L[͂L
			if((us & SW1_OFF_LONG) != 0){			// ꎞ(set switch)
				proc_set_rtc();						// v킹
				disp_clock_data(1);					// vf[^̕\i1=Sĕ\j
			}
		}

		if(rtc_1s_flg != 0){						// Pbo߂
			rtc_1s_flg = 0;
			led_grn_on_off(1);
			uc = rtc_read();						// RTC N ǂݏo
			if(uc != 0){
				rtc_error();						// qsbG[
			}
			uc = rtc_tempe_read();					// RTC xf[^ǂݏo
			if(uc != 0){
				rtc_error();						// qsbG[
			}
			disp_clock_data(0);						// vf[^̕\i0=ω̕\j
			led_grn_on_off(0);
		}

 		if(tim_10ms_infrared == 0){				// ԊORfqp^C}[ɗp
			tim_10ms_infrared = 50;
			toggle ^= 1;
			led_blu_on_off(toggle);

		}
	}
}
